"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NiceAuth = void 0;
var _ = require("lodash");
var iconv = require("iconv-lite");
var niceCrypto = require("../libs/crypto");
var nice_error_1 = require("../models/nice-error");
var NiceAuth = /** @class */ (function () {
    function NiceAuth(_options) {
        var _this = this;
        this._options = _options;
        this.generateRequestData = function (niceRequestParameter) { return __awaiter(_this, void 0, void 0, function () {
            var options, now, _issueToken, _issueCryptoToken, accessToken, _a, cryptoToken, siteCode, tokenVersionId, _b, symmetricKey, key, iv, hmacKey, requestData, encData, integrityValue;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        options = niceRequestParameter;
                        now = new Date();
                        _issueToken = niceCrypto.issueToken.bind(this);
                        _issueCryptoToken = niceCrypto.issueCryptoToken.bind(this);
                        return [4 /*yield*/, _issueToken()];
                    case 1:
                        accessToken = (_c.sent()).accessToken;
                        return [4 /*yield*/, _issueCryptoToken(options.requestno, accessToken, now)];
                    case 2:
                        _a = _c.sent(), cryptoToken = _a.cryptoToken, siteCode = _a.siteCode, tokenVersionId = _a.tokenVersionId;
                        _b = niceCrypto.generateSymmetricKey(options.requestno, cryptoToken, now), symmetricKey = _b.symmetricKey, key = _b.key, iv = _b.iv, hmacKey = _b.hmacKey;
                        options.sitecode = siteCode;
                        requestData = JSON.stringify(_.pickBy(options, _.identity)).trim();
                        encData = niceCrypto.encryptData(key, iv, requestData);
                        integrityValue = niceCrypto.hmacSha256(encData, hmacKey);
                        return [2 /*return*/, {
                                symmetricKey: symmetricKey,
                                formData: {
                                    tokenVersionId: tokenVersionId,
                                    encData: encData,
                                    integrityValue: integrityValue,
                                },
                            }];
                }
            });
        }); };
        this.decryptResultData = function (params) {
            try {
                if (!params.cipherData) {
                    throw new nice_error_1.DecryptDataError('invalid cipher data');
                }
                if (!params.integrityValue) {
                    throw new nice_error_1.DecryptDataError('invalid integrity value');
                }
                if (!params.symmetricKey) {
                    throw new nice_error_1.DecryptDataError('invalid symmetric key');
                }
                var key = params.symmetricKey.substring(0, 16);
                var iv = params.symmetricKey.substring(params.symmetricKey.length - 16);
                var hmacKey = params.symmetricKey.substring(0, 32);
                if (params.integrityValue !== niceCrypto.hmacSha256(params.cipherData, hmacKey)) {
                    throw new nice_error_1.DecryptDataError('integrity validation failed');
                }
                var decryptedBase64Text = niceCrypto.decryptData(key, iv, params.cipherData);
                var decodedPlainText = Buffer.from(decryptedBase64Text, 'base64');
                var decodedEucKrText = iconv.decode(decodedPlainText, 'euc-kr');
                var result_1 = niceCrypto.jsonParse(decodedEucKrText);
                _.forIn(result_1, function (value, key) {
                    if (key === 'utf8_name') {
                        result_1[key] = decodeURIComponent(value);
                    }
                    else if (key === 'name') {
                        result_1[key] = iconv.encode(value, 'utf-8').toString();
                    }
                    // else if (key === 'receivedata') {
                    //     const receivedata = this.jsonParse(value)
                    //     if (receivedata) {
                    //         result[key] = receivedata
                    //     }
                    // }
                });
                return result_1;
            }
            catch (e) {
                throw new nice_error_1.DecryptDataError((e && e.message) || 'Data decryption failed');
            }
        };
    }
    return NiceAuth;
}());
exports.NiceAuth = NiceAuth;
